#include "paging.h"

/***************stack***********************/
void *bootalloc(dword size)
{
 dword addr=kernelend;
 dword newend=kernelend+size;
 if(memorysize<=newend)
  return 0;
 kernelend=newend;
 return (void *)addr;
}

int isempty()
{
 if(s.top==-1)
  return 0;
 else
 return 1;
}

void push(dword index)
{
 s.top++;
 s.frames[s.top]=index;
}

dword pop()
{
 if (isempty()==0)
  return 0;
 else
 return s.frames[s.top--];

}

int size()
{
 return s.top;
}

void stack(dword nbframes)
{
 s.top=0;
 s.frames=(dword *)bootalloc(nbframes*sizeof(dword));
}

/**********************physical manager******************/

dword pagebase(dword address)
{
 return (address & page_mask);
}

dword pagetop(dword address)
{
 return pagebase(address) | page_size;
}

dword freepages()
{
 return size();
}

//free one physical page

void freeframe(dword index)
{
 push(index);
}

//alloc one physical page

dword allocframe(dword flags)
{
 byte *addr=(byte *)pop();
 dword val=((dword) addr)<<page_shift;
 if(addr && (flags & zero))
  memset(val,0,page_size);
 return val;
}

//push addresses from kernel end to the end of mem
//into the stack

void phmanager()
{
 int i;
 memtop=pagebase(memorysize+1);
 nbframes=memorysize/page_size;
 stack(nbframes);
 kerneltop=pagetop(kernelend);
 for(i=kerneltop;i<memtop;i+=page_size)
  push(i>>page_shift);
}

/**********************paging***********************/

//zone mapping in the last 4M of virtual address

const dword selfindex=255;	

pte_t *pagetables=(pte_t *)zone_mapping;
pde_t *pagedir=(pde_t *)page_dir;
pde_t *selfpde=(pde_t *)self_pde_map;

dword basedeneid,topdeneid;

//create page dirrectory
void makepde(pde_t *pde,physaddr_t paddr,dword flags)
{
 *pde=paddr | (flags & pde_flags_mask);
}

//create page table
void makepte(pte_t *pte,physaddr_t paddr,dword flags)
{
 *pte=paddr | (flags & pte_flags_mask);
}


dword pdeindex(virtaddr_t vaddr)
{
 return (vaddr >> 22);
}
dword pteindex(virtaddr_t vaddr)
{
 return ((vaddr >> 12)&0x3ff);
}
dword pageoffset(virtaddr_t vaddr)
{
 return (vaddr & vpage_mask);
}

//get the address of page directory
pde_t *pdeaddr(virtaddr_t vaddr)
{
 return (pagedir+pdeindex(vaddr));
}

//get the address of the page table
pte_t *pteaddr(virtaddr_t vaddr)
{
 return pagetables+(vaddr>>vpage_shift);
}

int issystemaddr(virtaddr_t vaddr)
{
 if((vaddr>=basedeneid && vaddr<topdeneid)||!vaddr)
  return 1;
 else
  return 0;
}

int ismapaddr(virtaddr_t vaddr)
{
 if (vaddr>=zone_mapping && vaddr<user_zone)
  return 1;
 else return 0;
}

void invlpage(virtaddr_t page)
{
 asm volatile("invlpg %0"::"m"(*(dword *)page));
}




pde_t *initpaging(physaddr_t kbase,physaddr_t ktop)
{
 int i=0;
 printf("Enable paging...");
 pte_t *kpte,*kpte_u;
 pde_t *kpde,*kpde_u,*kpd;
 physaddr_t paddr,paddr_u;
 pde_t *pd=(pde_t *)allocframe(0);
 for(i=2;i<1024;i++)
  pd[i]=0;
 for(paddr=kbase;paddr<ktop;paddr+=page_size,paddr_u=paddr)//Identity Mapping
  {
    kpde=pd+pdeindex(paddr);
    if(!(*kpde & present))
      {
        kpte=(pte_t *)allocframe(0);
        for(i=0;i<1024;i++)
          kpte[i]=0;
        makepde(kpde,(physaddr_t)kpte,present | write |user);
      }
    makepte(kpte+pteindex(paddr),paddr,present | write |user);
  }
 
 kpde_u=(pde_t *)allocframe(0);
 makepde(paddr_u,(physaddr_t)kpde_u, present | write |user);
 
 pbdr=((dword)kpde);
 pd_u=((dword)kpde_u);
 asm volatile(
                "movl %0, %%cr3	\n\t\
		 movl %%cr0, %%eax \n\t\
		 orl $0x80010000, %%eax \n\t\
		 movl %%eax, %%cr0	\n\t\
		 jmp	1f	\n\t\
		 1: \n\t"
		 ::"r"(pbdr):"memory","eax");

 
 

 basedeneid=kbase;
 topdeneid=ktop;
 printf("[done]\n");
 return pd;	    //page directory address
}
