#include "pm.h"
#include "defs.h"
/***************************General function*****************************/
void enable()
{
 __asm__ __volatile__ ("sti");
}
void disable()
{
 __asm__ __volatile__ ("cli");
}
/***********************************************************************/
extern gdt_flush();

/* Setup a descriptor in the Global Descriptor Table */
void gdt_set_gate(int num, unsigned long base, unsigned long limit, unsigned char access, unsigned char gran)
{
    /* Setup the descriptor base address */
 gdt[num].base_low = (base & 0xFFFF);
 gdt[num].base_middle = (base >> 16) & 0xFF;
 gdt[num].base_high = (base >> 24) & 0xFF;

    /* Setup the descriptor limits */
 gdt[num].limit_low = (limit & 0xFFFF);
 gdt[num].granularity = ((limit >> 16) & 0x0F);

    /* Finally, set up the granularity and access flags */
 gdt[num].granularity |= (gran & 0xF0);
 gdt[num].access = access;
}

void gdt_install()
{
 printf("Installing GDT...");
    /* Setup the GDT pointer and limit */
 unsigned long tmp;
 gp.limit = (sizeof(struct gdt_entry) * max_gdt_entrys) - 1;
 gp.base = &gdt;

    /* Our NULL descriptor 0x00 */
 gdt_set_gate(0, 0, 0, 0, 0);

    /* Code Segment 0x08 PL0*/
  gdt_set_gate(1, 0, 0xFFFF, ACS_CODE , 0xCF);

    /* Data Segment 0x10 PL0*/
  gdt_set_gate(2, 0,0xFFFF, ACS_DATA , 0xCF);
  
  /* 0x18 code segment descriptor PL3*/
  gdt_set_gate(3,0,0xFFFF,ACS_CODE | ACS_DPL_3 ,0xCF);
   
  /* 0x20 data segment descriptor PL3*/
  gdt_set_gate(4,0,0xFFFF,ACS_DATA | ACS_DPL_3,0xCF);

  /* -- 0x28 TSS for main() */
   gdt_set_gate(5,(dword)&tss[0],sizeof(TSS),ACS_TSS,0xCF);
  
  /* -- 0x30 TSS for task() */
   gdt_set_gate(6,(dword)&tss[1],sizeof(TSS),ACS_TSS,0xCF);
  

    /* Flush out the old GDT and install the new changes! */
  gdt_flush();
  printf("[Done]\n");
}
/******************************************************************/
extern void idt_load();

void idt_set_gate(int num, unsigned long base, unsigned short sel, unsigned char flags)
{
 idt[num].base_lo=(base & 0xFFFF);
 idt[num].base_hi=(base >> 16) & 0xFFFF;
 idt[num].sel=sel;
 idt[num].always0=0;
 idt[num].flags=flags;
}

void idt_install()
{
 printf("Installing IDT...");
 int i;
 idtp.limit=(sizeof(struct idt_entry) * 256) - 1;
 idtp.base=&idt;

 /*clear out the entire IDT to zero*/

  //memset(&idt,0,sizeof(struct idt_entry)*256);
   for(i=0;i<256;i++)
    {
     idt[i].base_lo=0;
     idt[i].sel=0;
     idt[i].always0=0;
     idt[i].flags=0;
     idt[i].base_hi=0;
    }
  idt_load();
 printf("[Done]\n");
}
/***************************************************************/
/*ISRs*/

extern void isr0();
extern void isr1();
extern void isr2();
extern void isr3();
extern void isr4();
extern void isr5();
extern void isr6();
extern void isr7();
extern void isr8();
extern void isr9();
extern void isr10();
extern void isr11();
extern void isr12();
extern void isr13();
extern void isr14();
extern void isr15();
extern void isr16();
extern void isr17();
extern void isr18();
extern void isr19();
extern void isr20();
extern void isr21();
extern void isr22();
extern void isr23();
extern void isr24();
extern void isr25();
extern void isr26();
extern void isr27();
extern void isr28();
extern void isr29();
extern void isr30();
extern void isr31();

unsigned char *exception_message[]=
{
 "Division By Zero",
 "Debug",
 "Non Maskable Interrupt",
 "Breakpoint",
 "Into Detect Overflow",
 "Out Of Bounds",
 "Invalid Opcode",
 "No Coprocessor",

 "Double Fault",
 "Coprocessor Segment Overrun",
 "Bad Tss",
 "Segment Not Present",
 "Stack Fault",
 "General Protection Fault",
 "Page Fault",
 "Unknown Interrupt",
 
 "Coprocessor Fault",
 "Alignment Check",
 "Reseved",
 "Reseved",
 "Reseved",
 "Reseved",
 "Reseved",

 "Reseved",
 "Reseved",
 "Reseved",
 "Reseved",
 "Reseved",
 "Reseved",
 "Reseved",
 "Reseved",
 
};

void fault_handler(struct regs *r)
{
 if(r->int_no<32)
   {
    printf(exception_message[r->int_no]);
    printf("\nException. System Halted!\n");
    for(;;);
   }
}

void isrs_install()
{
 printf("Installing ISRs...");
 idt_set_gate(0, (unsigned)isr0, 0x08, 0x8E);
 idt_set_gate(1, (unsigned)isr1, 0x08, 0x8E);
 idt_set_gate(2, (unsigned)isr2, 0x08, 0x8E);
 idt_set_gate(3, (unsigned)isr3, 0x08, 0x8E);
 idt_set_gate(4, (unsigned)isr4, 0x08, 0x8E);
 idt_set_gate(5, (unsigned)isr5, 0x08, 0x8E);
 idt_set_gate(6, (unsigned)isr6, 0x08, 0x8E);
 idt_set_gate(7, (unsigned)isr7, 0x08, 0x8E);

 idt_set_gate(8, (unsigned)isr8, 0x08, 0x8E);
 idt_set_gate(9, (unsigned)isr9, 0x08, 0x8E);
 idt_set_gate(10, (unsigned)isr10, 0x08, 0x8E);
 idt_set_gate(11, (unsigned)isr11, 0x08, 0x8E);
 idt_set_gate(12, (unsigned)isr12, 0x08, 0x8E);
 idt_set_gate(13, (unsigned)isr13, 0x08, 0x8E);
 idt_set_gate(14, (unsigned)isr14, 0x08, 0x8E);
 idt_set_gate(15, (unsigned)isr15, 0x08, 0x8E);

 idt_set_gate(16, (unsigned)isr16, 0x08, 0x8E);
 idt_set_gate(17, (unsigned)isr17, 0x08, 0x8E);
 idt_set_gate(18, (unsigned)isr18, 0x08, 0x8E);
 idt_set_gate(19, (unsigned)isr19, 0x08, 0x8E);
 idt_set_gate(20, (unsigned)isr20, 0x08, 0x8E);
 idt_set_gate(21, (unsigned)isr21, 0x08, 0x8E);
 idt_set_gate(22, (unsigned)isr22, 0x08, 0x8E);
 idt_set_gate(23, (unsigned)isr23, 0x08, 0x8E);

 idt_set_gate(24, (unsigned)isr24, 0x08, 0x8E);
 idt_set_gate(25, (unsigned)isr25, 0x08, 0x8E);
 idt_set_gate(26, (unsigned)isr26, 0x08, 0x8E);
 idt_set_gate(27, (unsigned)isr27, 0x08, 0x8E);
 idt_set_gate(28, (unsigned)isr28, 0x08, 0x8E);
 idt_set_gate(29, (unsigned)isr29, 0x08, 0x8E);
 idt_set_gate(30, (unsigned)isr30, 0x08, 0x8E);
 idt_set_gate(31, (unsigned)isr31, 0x08, 0x8E);
 printf("[Done]\n");
}

/**********************************************************/
/*IRQ*/

extern void irq0();
extern void irq1();
extern void irq2();
extern void irq3();
extern void irq4();
extern void irq5();
extern void irq6();
extern void irq7();
extern void irq8();
extern void irq9();
extern void irq10();
extern void irq11();
extern void irq12();
extern void irq13();
extern void irq14();
extern void irq15();

void *irq_routines[16] =
{
    0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0
};

void irq_install_handler(int irq, void (*handler)(struct regs *r))
{
    irq_routines[irq] = handler;
}

/* This clears the handler for a given IRQ */
void irq_uninstall_handler(int irq)
{
    irq_routines[irq] = 0;
}

/* Normally, IRQs 0 to 7 are mapped to entries 8 to 15. This
*  is a problem in protected mode, because IDT entry 8 is a
*  Double Fault! Without remapping, every time IRQ0 fires,
*  you get a Double Fault Exception, which is NOT actually
*  what's happening. We send commands to the Programmable
*  Interrupt Controller (PICs - also called the 8259's) in
*  order to make IRQ0 to 15 be remapped to IDT entries 32 to
*  47 */
void irq_remap(void)
{
    outportb(0x20, 0x11);
    outportb(0xA0, 0x11);
    outportb(0x21, 0x20);
    outportb(0xA1, 0x28);
    outportb(0x21, 0x04);
    outportb(0xA1, 0x02);
    outportb(0x21, 0x01);
    outportb(0xA1, 0x01);
    outportb(0x21, 0x0);
    outportb(0xA1, 0x0);
}

/*  remap the interrupt controllers */
void irq_install()
{
 printf("Installing IRQ...");
 irq_remap();
 idt_set_gate(32, (unsigned)irq0, 0x08, 0x8E);
 idt_set_gate(33, (unsigned)irq1, 0x08, 0x8E);
 idt_set_gate(34, (unsigned)irq2, 0x08, 0x8E);
 idt_set_gate(35, (unsigned)irq3, 0x08, 0x8E);
 idt_set_gate(36, (unsigned)irq4, 0x08, 0x8E);
 idt_set_gate(37, (unsigned)irq5, 0x08, 0x8E);
 idt_set_gate(38, (unsigned)irq6, 0x08, 0x8E);
 idt_set_gate(39, (unsigned)irq7, 0x08, 0x8E);

 idt_set_gate(40, (unsigned)irq8, 0x08, 0x8E);
 idt_set_gate(41, (unsigned)irq9, 0x08, 0x8E);
 idt_set_gate(42, (unsigned)irq10, 0x08, 0x8E);
 idt_set_gate(43, (unsigned)irq11, 0x08, 0x8E);
 idt_set_gate(44, (unsigned)irq12, 0x08, 0x8E);
 idt_set_gate(45, (unsigned)irq13, 0x08, 0x8E);
 idt_set_gate(46, (unsigned)irq14, 0x08, 0x8E);
 idt_set_gate(47, (unsigned)irq15, 0x08, 0x8E);
 printf("[Done]\n");
}


void irq_handler(struct regs *r)
{
    
    void (*handler)(struct regs *r);

    
    handler = irq_routines[r->int_no - 32];
    if (handler)
    {
        handler(r);
    }

    
    if (r->int_no >= 40)
    {
        outportb(0xA0, 0x20);
    }

    /*  send an EOI to the master
    *  interrupt controller  */
    outportb(0x20, 0x20);
}

/***********************************************************************/
void timer_phase(int hz)
{
 int div=1193180/hz;
 outportb(0x43,0x36);
 outportb(0x40, div & 0xFF);
 outportb(0x40, div >> 8);
}

extern int timer_ticks = 0;


void timer_handler(struct regs *r)
{
 /* Increment our 'tick count' */
    timer_ticks++;
	
    /* Every 18 clocks (approximately 1 second), we will
    *  display a message on the screen 
    if (timer_ticks % 18 == 0)
    {
        printf("One second has passed\n");
    }
 */
}

void timer_install()
{
    /* Installs 'timer_handler' to IRQ0 */
    irq_install_handler(0, timer_handler);
    //timer_phase(18);
}

/* works with milli secons 1000m=1second */

void delay(int t_ticks) 
{
 
    unsigned long eticks;
    t_ticks=(t_ticks * 18) / 1000;
    eticks = timer_ticks + t_ticks;
    while(timer_ticks < eticks);
 
}
/************************************************************************/
 






















































