#ifndef __PM_H
#define __PM_H

#include "scr.h"
/************************General function*******************************/
void enable();
void disable();

#define ACS_PRESENT	0X80		/* present segment */
#define ACS_CSEG	0X18		/* code segment */
#define ACS_DSEG	0X10		/* data segment */
#define ACS_CONIFORM	0X04
#define ACS_READ	0X02
#define ACS_WRITE	0X02
#define ACS_TSS_GATE	0X09
#define ACS_TSS		(ACS_PRESENT | ACS_TSS_GATE)

/* ready-made values */

#define ACS_CODE	(ACS_PRESENT | ACS_CSEG | ACS_READ)
#define ACS_DATA	(ACS_PRESENT | ACS_DSEG | ACS_WRITE) 
#define ACS_STACK       (ACS_PRESENT | ACS_DSEG | ACS_WRITE)	

#define ACS_DPL_0       0x00            /* descriptor privilege level #0 */
#define ACS_DPL_1       0x20            /* descriptor privilege level #1 */
#define ACS_DPL_2       0x40            /* descriptor privilege level #2 */
#define ACS_DPL_3       0x60            /* descriptor privilege level #3 */

#define ATTR_DEFAULT     0x40
/***********************************************************************/
/* Defines a GDT entry. We say packed, because it prevents the
*  compiler from doing things that it thinks is best: Prevent
*  compiler "optimization" by packing */

#define max_gdt_entrys	7

struct gdt_entry
{
    unsigned short limit_low;
    unsigned short base_low;
    unsigned char base_middle;
    unsigned char access;
    unsigned char granularity;
    unsigned char base_high;
} __attribute__((packed));

/* Special pointer which includes the limit: The max bytes
*  taken up by the GDT, minus 1. Again, this NEEDS to be packed */
struct gdt_ptr
{
    unsigned short limit;
    unsigned int base;
} __attribute__((packed));

/* Our GDT, with 8 entries, and finally our special GDT pointer */

struct gdt_entry gdt[max_gdt_entrys];
struct gdt_ptr gp;

/* This will be a function in start.asm. We use this to properly
*  reload the new segment registers */

void gdt_set_gate(int num, unsigned long base, unsigned long limit, unsigned char access, unsigned char gran);
void gdt_install();
/*********************************************************************/
/*IDT*/

struct idt_entry
{
 unsigned short base_lo;
 unsigned short sel;
 unsigned char always0;
 unsigned char flags;
 unsigned short base_hi;

} __attribute__((packed)); 

struct idt_ptr
{
 unsigned short limit;
 unsigned int base;

} __attribute__((packed));


struct idt_entry idt[256];
struct idt_ptr idtp;

void idt_install();
void idt_set_gate(int num, unsigned long base, unsigned short sel, unsigned char flags);
/************************************************************************/
/*ISRs*/

struct regs
{
 unsigned int gs,fs,es,ds;
 unsigned int edi,esi,ebp,esp,ebx,edx,ecx,eax;
 unsigned int int_no,err_code;
 unsigned int eip,cs,eflags,useresp,ss;
};

void fault_handler(struct regs *r);
void isrs_install();
/************************************************************************/
/*IRQ*/

void irq_install_handler(int irq, void (*handler)(struct regs *r));
void irq_uninstall_handler(int irq);
void irq_install();

/************************************************************************/
/*Timer*/

int timer_ticks ;

void timer_install();
void timer_handler(struct regs *r);
void timer_phase(int hz);
void delay(int t_ticks);
/******************************Task switching*****************************/
struct tss_t
{
 dword  link,
	esp0,
	ss0,
	esp1,
	ss1,
	esp2,
	ss2,
	cr3,
	eip,
	eflags,
	eax,
	ecx,
	edx,
	ebx,
	esp,
	ebp,
	esi,
	edi,
	es,
	cs,
	ss,
	ds,
	fs,
	gs,
	ldtr;
 word	trace,
	io_map_addr;
	
};

typedef struct tss_t TSS;

#define max_tasks	2

TSS tss[max_tasks];








#endif